unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

{  these constants are not defined in Windows
  SHELL32_STRING_ID_PIN_TO_TASKBAR = 5386;
  SHELL32_STRING_ID_PIN_TO_STARTMENU = 5381;
  SHELL32_STRING_ID_UNPIN_FROM_TASKBAR = 5387;
  SHELL32_STRING_ID_UNPIN_FROM_STARTMENU = 5382;
  }

const
  PROGRAM_PIN   = 5386; // ID   - "Pin this program to taskbar"
  PROGRAM_UNPIN = 5387; // ID   - "Unpin this program from taskbar"


type
  TfrmMain = class(TForm)
    btnGetApp: TButton;
    Edit1: TEdit;
    OpenDialog1: TOpenDialog;
    btnPin: TButton;
    btnUnPin: TButton;
    Label1: TLabel;
    procedure btnGetAppClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnPinClick(Sender: TObject);
    procedure btnUnPinClick(Sender: TObject);
  private
    { Private declarations }
    PinAppCaption: WideString;
    UnpinAppCaption: WideString;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  ComObj;

{$R *.dfm}

function GetWinResStr(resID: UINT): WideString;
var
 h: HMODULE;
 buf: array[0..512] of Byte;
 sys32dll: string;
 len: Integer;
begin
  Result := '';
  //      Shell32.dll
  ZeroMemory(@buf[0],512);
  len := GetSystemDirectoryA(PAnsiChar(@buf[0]),256);
  if (len = 0) then Exit;
  sys32dll := IncludeTrailingPathDelimiter(StrPas(PAnsiChar(@buf[0]))) + 'Shell32.dll';

  //  
  h := SafeLoadLibrary(sys32dll);
  if (h = 0) then Exit;
  try
    //      ID 
    ZeroMemory(@buf[0],512);
    len := LoadStringW(h,resID,PWideChar(@buf[0]),512);
    if (len >  0) then begin
      Result := PWideChar(@buf[0]);
      SetLength(Result,len);
    end;
  finally
    FreeLibrary(h);
  end;
end;


function AppyShellAppAction(appPath, appExeName: WideString; shellAction: WideString): Boolean;
var
  vShell, vFolder, vFolderItem, vItemVerbs : Variant;
  vPath, vApp, vAction: Variant;
  vi, vItemName: Variant;
  i,n: Integer;
  s: string;
begin
  Result  := False;
  s := appPath + appExeName;
  if not FileExists(s) then Exit;

  vPath   := appPath;
  vAction := shellAction;
  vApp    := appExeName;

  try
    try
      vShell := CreateOleObject('Shell.Application');
      vFolder := vShell.NameSpace(vPath);
      vFolderItem := vFolder.ParseName(vApp);
      vItemVerbs := vFolderItem.Verbs;

      n := vItemVerbs.Count;
      for i := 1 to n do
      begin
        vi := i;
        vItemName := vItemVerbs.Item(vi).Name;
        if (vItemName = vAction) then begin
          vItemVerbs.Item(vi).DoIt;
          Result := True;
          Break;
        end;
      end;

    except
      Result  := False;
    end;
  finally
    vShell      := Unassigned;
    vFolder     := Unassigned;
    vFolderItem := Unassigned;
    vItemVerbs  := Unassigned;
  end;
end;


procedure TfrmMain.btnGetAppClick(Sender: TObject);
begin
  if not OpenDialog1.Execute then Exit;
  if FileExists(OpenDialog1.FileName) then
    Edit1.Text := OpenDialog1.FileName;
end;

procedure TfrmMain.btnPinClick(Sender: TObject);
var
 appPath, appExeName: WideString;
 s: string;
begin
  s := Edit1.Text;
  if not FileExists(s) then begin
    ShowMessage(' "'+s+'"  ');
    Exit;
  end;

  appPath    := ExtractFilePath(s);
  appExeName := ExtractFileName(s);

  if not AppyShellAppAction(appPath, appExeName,PinAppCaption) then
    s := ' "'+PinAppCaption+'"  "'+appExeName+'"  '
  else
    s := '  ';

  Label1.Caption := s;
//  ShowMessage(s);
end;

procedure TfrmMain.btnUnPinClick(Sender: TObject);
var
 appPath, appExeName: WideString;
 s: string;
begin
  s := Edit1.Text;
  if not FileExists(s) then begin
    ShowMessage(' "'+s+'"  ');
    Exit;
  end;

  appPath    := ExtractFilePath(s);
  appExeName := ExtractFileName(s);

  if not AppyShellAppAction(appPath, appExeName,UnpinAppCaption) then
    s := ' "'+UnpinAppCaption+'"  "'+appExeName+'"  '
  else
    s := '  ';

  Label1.Caption := s;
//  ShowMessage(s);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  Len: Integer;
  s: string;
begin
  Edit1.Text := '';
  Len := GetWindowsDirectory(nil, 0);
  if Len > 0 then
  begin
    SetLength(s, Len);
    Len := GetWindowsDirectory(PChar(s), Len);
    SetLength(s, Len);
    if DirectoryExists(s) then begin
      OpenDialog1.InitialDir := s;
      Edit1.Text := IncludeTrailingPathDelimiter(s)+ 'notepad.exe';
    end;
  end;
  PinAppCaption   := GetWinResStr(PROGRAM_PIN);;
  UnpinAppCaption := GetWinResStr(PROGRAM_UNPIN);

  if (PinAppCaption <> '') then
    btnPin.Caption := PinAppCaption
  else
    btnPin.Enabled := False;

  if (UnpinAppCaption <> '') then
    btnUnPin.Caption := UnpinAppCaption
  else
    btnUnPin.Enabled := False;
end;

end.
